#ifndef __BTT_PEER_H
#define __BTT_PEER_H

/* libc */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
/* other libs */
#include <db.h>
#include <apr.h>
#include <apr_pools.h>
/* local */
#include <libbtutil.h>

/* flags */
#define BTT_PEER_SHIELD         0x01
#define BTT_PEER_COMPACT        0x02
#define BTT_PEER_NOPEERID	0x04
#define BTT_PEER_SHUNNED        0x08
#define BTT_PEER_PASSIVE        0x10
#define BTT_PEER_HTML           0x20

#define BTT_PEER_KEEP_FLAGS     (BTT_PEER_SHIELD | BTT_PEER_SHUNNED)

#define BTT_PEER_IS_SEED(p)     (p->left == 0)
#define BTT_PEER_IS_SHIELD(p)   ((p->flags & BTT_PEER_SHIELD) && BTT_PEER_IS_SEED(p))

#define BTT_PEER_IS_HIDDEN(p)   (BTT_PEER_IS_SHIELD(p) || (p->flags & BTT_PEER_SHUNNED))
#define BTT_PEER_WANTS_PEERS(p) ((p->num_want != 0) && !(BTT_PEER_IS_HIDDEN(p)))

typedef struct btt_peer_s
{
 char				peerid[BT_PEERID_LEN];
 char				infohash[BT_INFOHASH_LEN];
 char				ua[BT_SHORT_STRING];	/* User-Agent */
 char				event[BT_EVENT_LEN];	/* Last Event Receieved */

 unsigned char		flags;					/* flags */
 
 struct sockaddr_in	address;				/* reported IP address/port */
 struct sockaddr_in real_address;			/* real IP address/port */
 
 time_t				first_t;				/* first time we heard from peer */
 time_t				last_t;					/* last time we heard from peer */
 
 time_t				first_serve_t;			/* first time we gave this peer to another peer */
 time_t				last_serve_t;			/* last time we gave this peer to another peer */
 
 time_t				complete_t;				/* time that this peer reported it had 0 bytes left to d/l */
 time_t				return_interval;		/* last return_interval we served to this peer */

 u_int32_t			hits;					/* how many times this peer has hit /announce */
 u_int32_t			serves;					/* how mant times we have given this peer to someone else */
 int32_t			num_want;				/* how many peers this peer has asked for or "-1" for not specified */
 u_int32_t			num_got;				/* how many peers this peer got back after their last "/announce" request */

 u_int64_t			announce_bytes;			/* how many bytes we've sent this peer in "/announce" replies */
 u_int64_t			uploaded;				/* how much this peer has uploaded (bytes) */
 u_int64_t			downloaded;				/* how much this peer has downloaded (bytes) */
 u_int64_t			left;					/* how much this peer has left (bytes) */
} btt_peer;

typedef struct btt_peer_flag_s
{
 unsigned char		flag;
 char*				description;
 char*				config_name;
} btt_peer_flag;

extern const btt_peer new_btt_peer;
extern const btt_peer web_btt_peer;
extern const btt_peer_flag btt_peer_flags[];

extern int btt_peer_crossref(DB* dbp, const DBT* pkey, const DBT* pdata, DBT* skey);
extern int btt_peer_req2struct(apr_pool_t* p, const char* args, btt_peer* peer);

extern int btt_peer2compact(apr_pool_t* p, btt_peer* peer, char **result);
extern int btt_peer2nopeerid(apr_pool_t* p, btt_peer* peer, char **result);
extern int btt_peer2bencode(apr_pool_t* p, btt_peer* peer, char **result);
extern int btt_peer2tr(apr_pool_t* p, btt_peer* peer, char** result);


#endif /* __BTT_PEER_H */
