/******************************************************************************
 * Copyright (c) 2005 Eric Petit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *****************************************************************************/

#ifndef TR_BENCODE_H
#define TR_BENCODE_H 1

#include <apr_pools.h>
#include <sys/types.h>


#define BT_BENC_TYPE_INT  1
#define BT_BENC_TYPE_STR  2
#define BT_BENC_TYPE_LIST 4
#define BT_BENC_TYPE_DICT 8

typedef struct bt_bencode_s {
    char * begin;
    char * end;
    char   type;
    union
    {
        int64_t i;
        struct
        {
            int    i;
            char * s;
        } s;
        struct
        {
            int                 alloc;
            int                 count;
            struct bt_bencode_s * vals;
        } l;
    } val;
} bt_bencode;

extern int              bt_parse_bencode(apr_pool_t*pool, char * buf, bt_bencode * val, char ** end );
extern bt_bencode*      bt_find_bencode(bt_bencode * val, char * key );
extern const char*      bt_benc_error(int val);

#define BT_BENC_SUCCESS            0
#define BT_BENC_BAD_INT            1
#define BT_BENC_BAD_LIST           2
#define BT_BENC_BAD_DICT           3
#define BT_BENC_DICT_UNEVEN        4
#define BT_BENC_BAD_STRING         5
#define BT_BENC_BAD_STRING_LENGTH  6

#endif
