/******************************************************************************
 * Copyright (c) 2005 Eric Petit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *****************************************************************************/

#ifndef LIBBTUTIL_TYPES_BT_METAINFO_H
#define LIBBTUTIL_TYPES_BT_METAINFO_H

#include <apr.h>
#include <apr_pools.h>
#include <apr_network_io.h>

#include <libbtutil/const.h>

typedef struct bt_metainfo_finfo_s {
    uint64_t    length;
    char       name[BT_SHORT_STRING];
} bt_metainfo_finfo;

typedef struct bt_metainfo_s {
    /* Path to torrent */
    char               torrent[BT_SHORT_STRING];

    /* General info */
    char                hash[BT_INFOHASH_LEN];
    char               name[BT_SHORT_STRING];

    /* Tracker info */
    char               tracker_address[BT_SHORT_STRING];
    apr_port_t          tracker_port;
    char               tracker_announce[BT_SHORT_STRING];

    /* Pieces info */
    unsigned int        piece_size;
    unsigned int        piece_count;
    uint64_t            total_size;
    uint8_t*            pieces;

    /* Files info */
    int                 file_count;
    bt_metainfo_finfo*  files;
} bt_metainfo;


bt_metainfo* bt_parse_metainfo(apr_pool_t *, const char *);

#endif /* LIBBTUTIL_TYPES_BT_METAINFO_H */
